/**
 * FunctionIdx.java
 * Una clase que contiene las funciones que podrán ser
 * utilizadas desde la línea de comandos de H2
 * @version 1.0, 04/01/12
 * @author Santiago González
 */


import java.sql.*;
import java.sql.ResultSet;
import java.sql.Statement;
import org.h2.tools.SimpleResultSet;


public class FunctionIdx {


     /**
      * Función que permite ser invocar la creación de un índice espacial desde H2.
      * @param conn Conexión con la base de datos
      * @param tableName nombre de la tabla que contiene el campo geométrico a indexar
      * @param field nombre del campo geométrico a indexar
      * @return devuelve true o false en función del éxito o fracaso de la creación del índice
      */
    public static boolean st_createSpIndex(Connection conn, String tableName, String field) throws Exception {

		String tb = tableName;
		String fd = field;

		// Se realiza la carga en memoria Carga el contenido de la tabla que contiene los MBR en memoria en la estructura del ArbolR.
        return AuxIdx.createSpatialIndex(conn, tb, fd);	
	
    }
    

     /**
      * Función que permite realizar la búsqueda de todos los elementos contenidos completamente
      * dentro de un rectángulo dado
      * @param conn Conexión con la base de datos
      * @param tableName nombre de la tabla que contiene el campo geométrico a indexar
      * @param field nombre del campo geométrico indexado
      * @param XMin,YMin,XMax,YMax valores del rectángulo MBR sobre el que se realiza la búsqueda de elementos
      * @return devuelve el identificador de los distintos elementos incluidos dentro del MBR de entrada
      */

    
        public static SimpleResultSet st_contains_IDX(Connection conn, String tableName, String field, float Xmin, float Ymin, float Xmax,float Ymax) throws Exception {

                
               	String tb = tableName;
		String fd = field;

                SimpleResultSet rs = new SimpleResultSet();
        
		rs = AuxIdx.getContainFunction(conn, tb, fd, Xmin, Ymin, Xmax, Ymax );
		
		return rs;
        }


     /**
      * Función que permite realizar la búsqueda de todos los elementos contenidos completa o parcialmente
      * dentro de un rectángulo dado
      * @param conn Conexión con la base de datos
      * @param tableName nombre de la tabla que contiene el campo geométrico a indexar
      * @param field nombre del campo geométrico indexado
      * @param XMin,YMin,XMax,YMax valores del rectángulo MBR sobre el que se realiza la búsqueda de elementos
      * @return devuelve el identificador de los distintos elementos incluidos dentro del MBR de entrada
      */

    
        public static SimpleResultSet st_intersects_IDX(Connection conn, String tableName, String field, float Xmin, float Ymin, float Xmax,float Ymax) throws Exception {
            
                
                String tb = tableName;
		String fd = field;
		
                SimpleResultSet rs = new SimpleResultSet();
		rs = AuxIdx.getIntersectFunction(conn, tb, fd, Xmin, Ymin, Xmax, Ymax );

		return rs;		
	
	}
        
         
     /**
      * Función que permite realizar la comprobación de si dos geometrías pertenecientes a dos índices espaciales
      * se encuentran contenidas total o parcialmente entre ellas, utilizando sus respectivos índices espaciales
      * como elemento base en la búsqueda de sus datos.
      * @param conn Conexión con la base de datos
      * @param geom valor del campo geométrico a comparar
      * @param tableName nombre de la tabla que contiene el campo geométrico
      * @param field nombre del campo geométrico
      * @param geom2 valor del segundo campo geométrico a comparar
      * @param tableName2 nombre de la tabla que contiene el campo geométrico 2
      * @param field2 nombre del campo geométrico 2
      * @return devuelve true en caso de que las geometrías tengan intersección o false en caso contrario
      */
    
        public static boolean _st_intersects(Connection conn,byte[] geom1, String tableName1, String field1, byte[] geom2, String tableName2, String field2) throws Exception {

                //Obtenemos los MBR de cada geometría
                Rectangle r = new Rectangle(es.upv.jaspa.SQL.ST_XMin(geom1).floatValue(),es.upv.jaspa.SQL.ST_YMin(geom1).floatValue(),es.upv.jaspa.SQL.ST_XMax(geom1).floatValue(),es.upv.jaspa.SQL.ST_YMax(geom1).floatValue());
                Rectangle r2 = new Rectangle(es.upv.jaspa.SQL.ST_XMin(geom2).floatValue(),es.upv.jaspa.SQL.ST_YMin(geom2).floatValue(),es.upv.jaspa.SQL.ST_XMax(geom2).floatValue(),es.upv.jaspa.SQL.ST_YMax(geom2).floatValue());

                // Ejecutamos la intersección de ambos MBR utilizando los índices espaciales respectivos
                return AuxIdx.getIntersectMBRFunction(conn, tableName1, field1,r,tableName2, field2,r2);
 
        }
        
     /**
      * Función que realiza el lanzamiento de las acciones de creación automáticas de los índices espaciales en el sistema
      * @param conn Conexión con la base de datos
      */

            public static void st_init_IDX(Connection conn) throws Exception {
           
              AuxIdx.initIDX(conn);
              // Se crean los alias asociados a las nuevas funciones creadas: ST_CREATESPINDEX,ST_DROPSPINDEX,ST_LOADSPINDEX y ST_USESPINDEX
			try {		
				Statement stmt12 = conn.createStatement();						
				stmt12.execute("CREATE ALIAS IF NOT EXISTS JASPA.ST_CREATE_SPIDX FOR \"FunctionIdx.st_createSpIndex\" ");	
				stmt12.close();						
			} catch(SQLException ex) {
				System.err.println("SQLException: " + ex.getMessage());
			}
			
			try {		
				Statement stmt12 = conn.createStatement();						
				stmt12.execute("CREATE ALIAS IF NOT EXISTS JASPA.ST_CONTAINS_SPIDX FOR \"FunctionIdx.st_contains_IDX\" ");	
				stmt12.close();						
			} catch(SQLException ex) {
				System.err.println("SQLException: " + ex.getMessage());
			}	
			
			try {		
				Statement stmt12 = conn.createStatement();						
				stmt12.execute("CREATE ALIAS IF NOT EXISTS JASPA.ST_INTERSECTS_SPIDX FOR \"FunctionIdx.st_intersects_IDX\" ");	
				stmt12.close();						
			} catch(SQLException ex) {
				System.err.println("SQLException: " + ex.getMessage());
			}
                        try {		
				Statement stmt12 = conn.createStatement();						
				stmt12.execute("CREATE ALIAS IF NOT EXISTS JASPA._ST_INTERSECTS FOR \"FunctionIdx._st_intersects\" ");	
				stmt12.close();						
			} catch(SQLException ex) {
				System.err.println("SQLException: " + ex.getMessage());
			}

			
			System.out.println();
			System.out.println("Los alias ST_CREATE_SPIDX,ST_CONTAINS_SPIDX,STST_INTERSECTS_SPIDX y _ST_INTERSECTS se han creado satisfactoriamente y pueden ser invocados desde la consola de H2");
			System.out.println();
                
        }
}
