/**
 * TirggerIdx.java
 * Una clase que contiene las funciones asociadas a los lanzadores
 * que permiten mantener los índices espaciales actualizados de forma
 * automática.
 * Esta clase implementa la clase TRIGGERS de H2
 * @version 1.0, 04/01/12
 * @author Santiago González
 */

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import org.h2.api.Trigger;


public class TriggerIdx implements Trigger {

    private static final Map<String, TriggerIdx> TRIGGERS = Collections.synchronizedMap(new HashMap<String, TriggerIdx>());
    private String triggerData;
    private String tbname;
    private int trtype;
    private int pos;
    private int treepos;


    /**
     * Método para convertir una array de bytes en una cadena
     * @param in valor a convertir
     * @return devuelve una cadena de texto
     */ 
	public static String byteArrayToString(byte[] in){
		char out[] = new char[in.length * 2];
		for (int i = 0; i < in.length; i++) {
			out[i * 2] = "0123456789ABCDEF".charAt((in[i] >> 4) & 15);
			out[i * 2 + 1] = "0123456789ABCDEF".charAt(in[i] & 15);
		}
		return new String(out);
	}		

	
    /**
     * Método para inicia la acción adecuada en función del lanzador
     * @param conn Conexión con la base de datos
     * @param schemaName nombre del esquema donde se encuentra la tabla a actualizar
     * @param triggerName nombre del lanzador a ejecutar
     * @param tableName nombre de la tabla a actualizar
     * @param before contiene el valor antiguo del elemento a actualizar
     * @param type tipo de operación a realizar (INSERT, DELETE, UPDATE, SELECT)
     * @return devuelve una cadena de texto
     */

    public void init(Connection conn, String schemaName,
        String triggerName, String tableName, boolean before,
            int type) throws SQLException {
        TRIGGERS.put(getPrefix(conn) + triggerName, this);
		
	tbname = tableName;
        trtype = type;	
			
	ResultSet rs = conn.getMetaData().getColumns(null,schemaName, tableName, null);
	pos = 0;
        String column="";
		
        // Se localiza en la estructura de la tabla la posicion del campo que contiene los datos geometricos
	while (rs.next()) {
		column = rs.getString("COLUMN_NAME");
		if (column.equals(triggerData)) break;
		else pos++;
		}
	pos--;		
	System.out.println("Trigerdata inicio:" + triggerData + " COL:" + column + " Pos:" + pos );		
	System.out.println("Tabla en init:" + tbname + " Campo:" + column );		
    }


    /**
     * Método que activa la acción adecuada en función del lanzador
     * @param conn Conexión con la base de datos
     * @param old contiene el valor antiguo del elemento a actualizar
     * @param row contiene el valor actual del elemento a actualizar
     * @return devuelve una cadena de texto
     */

    public void fire(Connection conn, Object[] old, Object[] row) throws SQLException{
		
	// Muestra el tipo de operacion realizada sobre la tabla que est� asociada al trigger
	System.out.println("Tipo de operacion:"+trtype);
	System.out.println("Trigerdata disparo:"+triggerData);		
	
	// Se ha detectado una inserción en la tabla de datos
	if (trtype==1) {		
		String geomFromTrigger	=  byteArrayToString((byte[]) row[pos]);
		try {			
			Statement stmt = conn.createStatement();
                        ResultSet rs = stmt.executeQuery("SELECT ST_xmin(\'" + geomFromTrigger + "\') as minx,ST_ymin(\'" + geomFromTrigger + "\') as miny,ST_xmax(\'" + geomFromTrigger + "\') as maxx,ST_ymax(\'" + geomFromTrigger + "\') as maxy");			
                        rs.next();
                        PreparedStatement prep = conn.prepareStatement("INSERT INTO SpatialDataMBRTable_"+ tbname + "_" + triggerData+" VALUES("+row[0]+","+rs.getFloat(1)+","+rs.getFloat(2)+","+rs.getFloat(3)+","+rs.getFloat(4)+")");		
                        prep.execute();
                        prep.close();
                        Rectangle r = new Rectangle(rs.getFloat(1),rs.getFloat(2),rs.getFloat(3),rs.getFloat(4));
                        int i = Integer.parseInt(row[0].toString());
                        AuxIdx.updateIDX(trtype,treepos,i,r); 
		} catch(SQLException ex) {
				System.err.println("SQLException: " + ex.getMessage());
		}					
	}

	// Se ha detectado una actualización en la tabla de datos
	if (trtype==2) {		
		String geomFromTrigger	=  byteArrayToString((byte[]) row[pos]);
		try {				
			Statement stmt = conn.createStatement();
                        ResultSet rs = stmt.executeQuery("SELECT ST_xmin(\'" + geomFromTrigger + "\') as minx,ST_ymin(\'" + geomFromTrigger + "\') as miny,ST_xmax(\'" + geomFromTrigger + "\') as maxx,ST_ymax(\'" + geomFromTrigger + "\') as maxy");			
                        rs.next();
                        PreparedStatement prep = conn.prepareStatement("UPDATE SpatialDataMBRTable_"+ tbname + "_" + triggerData+" SET (XMIN="+rs.getFloat(1)+",YMIN ="+rs.getFloat(2)+",XMAX = "+rs.getFloat(3)+",YMAX ="+rs.getFloat(4)+") WHERE SGID=" + row[0]+")");		
                        prep.execute();
                        prep.close();
                        Rectangle r = new Rectangle(rs.getFloat(1),rs.getFloat(2),rs.getFloat(3),rs.getFloat(4));
                        int i = Integer.parseInt(row[0].toString());
                        AuxIdx.updateIDX(trtype,treepos,i,r); 
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		}		
		}

		// Se ha detectado la eliminación de un registro en el fichero de datos
	if (trtype==4) {	
		String geomFromTrigger	=  byteArrayToString((byte[]) old[pos]);
                try {	
			Statement stmt = conn.createStatement();
                        ResultSet rs = stmt.executeQuery("SELECT ST_xmin(\'" + geomFromTrigger + "\') as minx,ST_ymin(\'" + geomFromTrigger + "\') as miny,ST_xmax(\'" + geomFromTrigger + "\') as maxx,ST_ymax(\'" + geomFromTrigger + "\') as maxy");			
                        rs.next();
                        PreparedStatement prep = conn.prepareStatement("DELETE FROM SpatialDataMBRTable_"+ tbname + "_" +triggerData+" WHERE SGID="+ old[0]);	
                        prep.execute();
                        prep.close();
                        Rectangle r = new Rectangle(rs.getFloat(1),rs.getFloat(2),rs.getFloat(3),rs.getFloat(4));
                        int i = Integer.parseInt(old[0].toString());
                        AuxIdx.updateIDX(trtype,treepos,i,r); 	
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		}				
		}
        
		System.out.println("Tabla en fire:" + tbname + " Campo:" + triggerData );			
    }

    public void close() {
        // ignore
    }

    public void remove() {
        // ignore
    }

    /**
     * Call this method to change a specific trigger.
     *
     * @param conn the connection
     * @param trigger the trigger name
     * @param data the data
     */
    public static void setTriggerData(Connection conn, String trigger, String data) throws SQLException {
        TRIGGERS.get(getPrefix(conn) + trigger).triggerData = data;
    }

    private static String getPrefix(Connection conn) throws SQLException {
        Statement stat = conn.createStatement();
        ResultSet rs = stat.executeQuery("call ifnull(database_path() || '_', '') || database() || '_'");
        rs.next();
        return rs.getString(1);
    }

}